/*
 * Adafruit Odometer
 * Modified November 2021
 * 
  Based on the codes:   
 * Arduino Odometer by Cesare Brizio | CC by-sa-nc (this sketch is in the public domain) (http://www.cesarebrizio.it/Arduino/Speedometer.html)
 * Basic speedometer using millis() by John Boxall | CC by-sa-nc (http://tronixstuff.com/tutorials)
 * Debounce tutorial code by Arduino |GPL General Public License (https://www.arduino.cc/en/Tutorial/BuiltInExamples/Debounce)


----------------------------------------------------------------------------------
Implementation:
  - The bike wheel perimeter is measured marking a part of the tyre which matches with a mark on the ground.
  - One complete turn is done as straight as possible,
  - when the mark on the tyre touches the ground again a second mark is defined on the ground. 
  - Finally, with a steel tape measure the distance between the two marks, which represents the wheel perimeter in meters.

 Hardware
 - A reed switch sensor is required which must be connected to the microcontroller with a pull up resistor and a capacitor
 - Aditionally some magnets are required. Those will be placed on the front wheel spokes 
 - The reset of counts will be enabled with the reset button of the microcontroller 
 
  ==========================
  The circuit for 16 x 2 LCD display
  4-wire dialogue
  ==========================
The microcontroller used was Adafruit m0 feather
 
 * LCD RS pin to digital pin 6 ->  A register select (RS) pin that controls where in 
  the LCD's memory you're writing data on. You can select either the data register, 
  which holds what goes on the screen, or an instruction register, which is where 
  the LCD's controller looks for instructions on what to do next.
 
 * LCD Enable pin to digital pin 5 -> An Enable pin that enables writing to the registers

 8 data pins (D0 -D7). The states of these pins (high or low) are the bits that you're writing
 on a register when you write, or the values you're reading when you read. In this case we only use 4 wire dialogue 
 
 * LCD D4 pin to digital pin 9
 * LCD D5 pin to digital pin 10
 * LCD D6 pin to digital pin 11
 * LCD D7 pin to digital pin 12 
 
 * LCD R/W pin to ground -> A Read/Write (R/W) pin that selects reading mode or writing mode
 
 * LCD VSS pin to ground
 * LCD VCC pin to 5V -> In this case it is conected to the USB pin suply 
 
 * wiper to LCD VO pin (pin 3)
 *  LED Backlight (Bklt+ and BKlt-) pins that you can use to power the LCD, 
    control the display contrast, and turn on and off the LED backlight, respectively
*/


// include the library code:
#include <LiquidCrystal.h>

// initialize the library with the numbers of the interface pins
const int rs = 6, en = 5, d4 = 9, d5 = 10, d6 = 11, d7 = 12;

LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

//-const int buttonPin = 4;     // the number of the pushbutton pin -> A push button can be placed in pin 4
const int SensorPin = A1;       // the number of the Magnetic reed switch
//int buttonState = 0;          // variable for reading the pushbutton status
int Ready =0;                   // first switching 

float start, finished;          // Front wheel perimeter = 2.126 
float circMetric=0.5315;        // Split wheel perimeter(in meters) -> 4 magnets = 2.126/4 = 0.5315
//float circImperial;           // using 1 kilometer = 0.621371192 miles
//float speedk, speedm;         // holds calculated speed vales in metric and imperial
float cont =0.000;              // The counter most be initialised in 0
float distancek=0;              // holds calculated distance values in meters

int SensorState=HIGH;               // the current reading from the input pin
int lastSensorState = HIGH;         // the previous reading from the input pin
unsigned long lastDebounceTime= 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 10;    // the debounce time increases if the output flickers
int reading1 = digitalRead(SensorPin);

// Press Button Variables (Not used in this program)
int buttonState=HIGH;              // the current reading from the input pin
int lastButtonState = HIGH;       // the previous reading from the input pin
//unsigned long lastDebounceTime2 = 0;  // the last time the output pin was toggled
//unsigned long debounceDelay2 = 10;    // the debounce time increases if the output flickers
//int reading2 = digitalRead(buttonPin);




void setup() {
  pinMode(SensorPin, INPUT);    // Define Reed sensor pin as an input


  start=millis();  // returns the number of miliseconds since the Adafruit started to execute after reseting or turning on.
                   // This number will overgrow (become 0) after approximately 50 days. Returns the miliseconds into an unsigned long value.
   

  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);

  // Print a transitory message to the LCD.
  lcd.print("Odometer        ");
  delay(1500); //just to allow you to read the initialization message

  // top serial dialogue speed improves precision
  Serial.begin(115200); 
}

void loop() {
// The loop will be interrupted by the sensor each time the 
  // magnet passes near the sensor, in other words once per revolution
    lcd.setCursor(0,0);  // write the text on the top left
    lcd.print("Distanza Totale");
    
    if(Ready==1 && cont==1)  // Starts taking into account the first event as reference 
    {
     
      lcd.setCursor(0,1);
      lcd.print("                ");
      lcd.setCursor(0,1);
      lcd.print("READY!          ");
      
    }
    
    if(Ready==1 && cont>1)
    {
      //bottom line the 16 char, 2 lines display - time data
      lcd.setCursor(0,1);
      lcd.print("                ");
      lcd.setCursor(0,1);
      lcd.print(float(distancek));
      lcd.print("m           ");
      lcd.setCursor(9,1);
      lcd.print(int(cont-1 )); // starts counting events once the first reference event happens 
    }
  // adjust for personal preference to minimise flicker
  //delay(250); 

  //SENSOR -> Not used
  // Code: https://www.arduino.cc/en/Tutorial/BuiltInExamples/Debounce
        reading1 = digitalRead(SensorPin);
        if (reading1 != lastSensorState) // reset the debouncing timer
        {
          lastDebounceTime = millis();
        }

        if ((millis() - lastDebounceTime) > debounceDelay) {
          // whatever the reading is at, it's been there for longer than the debounce
          // delay, so take it as the actual current state:
      
          // if the button state has changed:
          if (reading1 != SensorState) {
            SensorState = reading1;
      
            // only toggle the LED if the new button state is HIGH
            if (SensorState == LOW) {
              Ready=1;
              cont= cont+1;  
              distancek= circMetric*(cont-1);
            }
          }
        }  
        lastSensorState = reading1;
}
